<?php

namespace AudentioForumConnect\Actions;

use AudentioForumConnect\Client;
use AudentioForumConnect\HookInterface;
use AudentioForumConnect\HookTrait;

/**
 * Class AdminMenu
 * @package AudentioForumConnect\Actions
 */
class AdminMenu implements HookInterface
{
    use HookTrait;

    public static function callback()
    {
        self::registerSettings();
    }

    public static function registerSettings()
    {
        $AudentioForumConnectBridgeTitle = \AudentioForumConnect::phrase('Article and Forum Connect: XenForo and WP');
        add_options_page(
            $AudentioForumConnectBridgeTitle,
            $AudentioForumConnectBridgeTitle,
            'manage_options',
            'article-forum-connect',
            [self::class, 'renderSettingsPage']
        );

        $option = get_option('wp_forum_connect', []);
        if (empty($option['commenting_mode'])) {
            $option['commenting_mode'] = 'linked_threads';
            update_option('wp_forum_connect', $option);
        }
        if (empty($option['integration_type'])) {
            $option['integration_type'] = 'session_cookie';
            update_option('wp_forum_connect', $option);
        }
        if (empty($option['connected_user_method'])) {
            $option['connected_user_method'] = 'login';
            update_option('wp_forum_connect', $option);
        }

        if (empty($option['api_key'])) {

            add_settings_section(
                'article-forum-connect-initial-setup-step-1',
                \AudentioForumConnect::phrase('Initial Setup'),
                null,
                'article-forum-connect'
            );

            add_settings_field(
                'article-forum-connect-xf-url',
                \AudentioForumConnect::phrase('XenForo URL'),
                [self::class, 'renderSettingsFieldXFUrl'],
                'article-forum-connect',
                'article-forum-connect-initial-setup-step-1'
            );

            add_settings_section(
                'article-forum-connect-initial-setup-step-2',
                '',
                [self::class, 'renderSetupFieldWPAPIKey'],
                'article-forum-connect'
            );

            add_settings_field(
                'article-forum-connect-wp-api-key',
                \AudentioForumConnect::phrase('WP API key'),
                [self::class, 'renderSettingsFieldWPAPIKey'],
                'article-forum-connect',
                'article-forum-connect-initial-setup-step-2'
            );

            if (!empty($option['xf_url'])) {

                add_settings_section(
                    'article-forum-connect-initial-setup-step-3',
                    '',
                    null,
                    'article-forum-connect'
                );

                add_settings_field(
                    'article-forum-connect-commenting-mode',
                    \AudentioForumConnect::phrase('Commenting mode'),
                    [self::class, 'renderSettingsFieldCommentingMode'],
                    'article-forum-connect',
                    'article-forum-connect-initial-setup-step-3'
                );

                add_settings_field(
                    'article-forum-connect-integration-type',
                    \AudentioForumConnect::phrase('Integration method'),
                    [self::class, 'renderSettingsFieldIntegrationType'],
                    'article-forum-connect',
                    'article-forum-connect-initial-setup-step-3'
                );

                add_settings_field(
                    'article-forum-connect-api-key',
                    \AudentioForumConnect::phrase('XenForo API Key'),
                    [self::class, 'renderSettingsFieldApiKey'],
                    'article-forum-connect',
                    'article-forum-connect-initial-setup-step-3'
                );

            }

        } else {

            add_settings_section(
                'article-forum-connect-integration-setup',
                 \AudentioForumConnect::phrase('Integration Setup'),
                null,
                'article-forum-connect'
            );

            add_settings_field(
                'article-forum-connect-commenting-mode',
                \AudentioForumConnect::phrase('Commenting mode'),
                [self::class, 'renderSettingsFieldCommentingMode'],
                'article-forum-connect',
                'article-forum-connect-integration-setup'
            );

            add_settings_field(
                'article-forum-connect-integration-type',
                \AudentioForumConnect::phrase('Integration method'),
                [self::class, 'renderSettingsFieldIntegrationType'],
                'article-forum-connect',
                'article-forum-connect-integration-setup'
            );

            add_settings_field(
                'article-forum-connect-connected-user-method',
                \AudentioForumConnect::phrase('Connected User Method'),
                [self::class, 'renderSettingsFieldConnectedUserMethod'],
                'article-forum-connect',
                'article-forum-connect-integration-setup',
                [
                    'class' => 'connected-user-method-container'
                ]
            );

            add_settings_field(
                'article-forum-connect-use-native-comments',
                \AudentioForumConnect::phrase('Native comments'),
                [self::class, 'renderSettingsFieldNativeComments'],
                'article-forum-connect',
                'article-forum-connect-integration-setup',
                [
                    'class' => 'use-native-comments-container'
                ]
            );

            add_settings_section(
                'article-forum-connect-AudentioForumConnectdetails',
                \AudentioForumConnect::phrase('XenForo API Details'),
                null,
                'article-forum-connect'
            );

            add_settings_field(
                'article-forum-connect-xf-url',
                \AudentioForumConnect::phrase('XenForo URL'),
                [self::class, 'renderSettingsFieldXFUrl'],
                'article-forum-connect',
                'article-forum-connect-AudentioForumConnectdetails'
            );

            add_settings_field(
                'article-forum-connect-api-key',
                \AudentioForumConnect::phrase('XenForo API Key'),
                [self::class, 'renderSettingsFieldApiKey'],
                'article-forum-connect',
                'article-forum-connect-AudentioForumConnectdetails'
            );

            add_settings_section(
                'article-forum-connect-wpapidetails',
                 \AudentioForumConnect::phrase('WP API Details'),
                null,
                'article-forum-connect'
            );

            add_settings_field(
                'article-forum-connect-wp-api-key',
                \AudentioForumConnect::phrase('WP API key'),
                [self::class, 'renderSettingsFieldWPAPIKey'],
                'article-forum-connect',
                'article-forum-connect-wpapidetails'
            );

            add_settings_field(
                'article-forum-connect-wp-api-user',
                \AudentioForumConnect::phrase('WP API user'),
                [self::class, 'renderSettingsFieldWPAPIUser'],
                'article-forum-connect',
                'article-forum-connect-wpapidetails'
            );

            if ($option['commenting_mode'] != 'auth_only') {

                add_settings_section(
                    'article-forum-connect-threaddefaults',
                     \AudentioForumConnect::phrase('Thread Defaults'),
                    null,
                    'article-forum-connect'
                );

                add_settings_field(
                    'article-forum-connect-create-thread',
                    \AudentioForumConnect::phrase('Create thread'),
                    [self::class, 'renderSettingsFieldCreateThread'],
                    'article-forum-connect',
                    'article-forum-connect-threaddefaults'
                );

                add_settings_field(
                    'article-forum-connect-default-forum',
                    \AudentioForumConnect::phrase('Default forum'),
                    [self::class, 'renderSettingsFieldDefaultForum'],
                    'article-forum-connect',
                    'article-forum-connect-threaddefaults'
                );

                add_settings_field(
                    'article-forum-connect-default-user',
                    \AudentioForumConnect::phrase('Default user'),
                    [self::class, 'renderSettingsFieldDefaultUser'],
                    'article-forum-connect',
                    'article-forum-connect-threaddefaults'
                );

            }

            add_settings_section(
                'article-forum-connect-advanced',
                 \AudentioForumConnect::phrase('Advanced Settings'),
                null,
                'article-forum-connect'
            );

            add_settings_field(
                'article-forum-connect-always-use-defaults',
                \AudentioForumConnect::phrase('Always use thread defaults'),
                [self::class, 'renderSettingsFieldAlwaysUseDefaults'],
                'article-forum-connect',
                'article-forum-connect-advanced'
            );

            add_settings_field(
                'article-forum-connect-hide-article-discussion-link',
                \AudentioForumConnect::phrase('Hide article discussion link'),
                [self::class, 'renderSettingsFieldHideArticleDiscussionLink'],
                'article-forum-connect',
                'article-forum-connect-advanced'
            );

            add_settings_field(
                'article-forum-connect-email-gravatar',
                \AudentioForumConnect::phrase('Use email address for gravatar'),
                [self::class, 'renderSettingsFieldEmailGravatar'],
                'article-forum-connect',
                'article-forum-connect-advanced'
            );

            if ($option['commenting_mode'] != 'auth_only') {

                add_settings_field(
                    'article-forum-connect-post-thread-id-meta-key',
                    \AudentioForumConnect::phrase('Legacy thread ID meta key'),
                    [self::class, 'renderSettingsFieldPostThreadIdMetaKey'],
                    'article-forum-connect',
                    'article-forum-connect-advanced'
                );

            }

            add_settings_field(
                'article-forum-connect-xf-cookie-prefix',
                \AudentioForumConnect::phrase('XF cookie prefix'),
                [self::class, 'renderSettingsFieldCookiePrefix'],
                'article-forum-connect',
                'article-forum-connect-advanced',
                [
                    'class' => 'xf-cookie-prefix-container'
                ]
            );

            add_settings_field(
                'article-forum-connect-default-user-id',
                \AudentioForumConnect::phrase('Default forum user ID'),
                [self::class, 'renderSettingsFieldDefaultForumUserId'],
                'article-forum-connect',
                'article-forum-connect-advanced'
            );

            add_settings_field(
                'article-forum-connect-session-auth-client-key',
                \AudentioForumConnect::phrase('Audentio API session auth client key'),
                [self::class, 'renderSettingsFieldSessionAuthClientKey'],
                'article-forum-connect',
                'article-forum-connect-advanced',
                [
                    'class' => 'xf-session-auth-client-key-container'
                ]
            );

            add_settings_field(
                'article-forum-connect-auto-open-frame',
                \AudentioForumConnect::phrase('Auto-open iframe'),
                [self::class, 'renderSettingsFieldAutoOpenIframe'],
                'article-forum-connect',
                'article-forum-connect-advanced',
                [
                    'class' => 'xf-auto-open-frame-container'
                ]
            );

            add_settings_field(
                'article-forum-connect-auto-open-frame-query-string',
                \AudentioForumConnect::phrase('Auto-open iframe by query string'),
                [self::class, 'renderSettingsFieldAutoOpenIframeQueryString'],
                'article-forum-connect',
                'article-forum-connect-advanced',
                [
                    'class' => 'xf-auto-open-frame-query-string-container'
                ]
            );

        }

        register_setting('article-forum-connect', 'wp_forum_connect', [self::class, 'validateAPISettings']);

        register_setting('article-forum-connect', 'wp_forum_connect_regen_api_key', [
            'type' => 'boolean',
            'description' => 'API Key regenerate checkbox',
            'default' => true,
            'sanitize_callback' => [self::class, 'resetWPAPIKey']
        ]);
    }

    public static function renderSettingsFieldXFUrl()
    {
        $options = get_option('wp_forum_connect', []);
        printf(
            '<input type="url" name="%s" value="%s" class="regular-text code" /><p class="description">%s</p>',
            esc_attr('wp_forum_connect[xf_url]'),
            esc_attr($options['xf_url'] ?? ''),
            \AudentioForumConnect::phrase('Your XenForo board URL as defined under <code>ACP > Setup > Basic options > Board URL</code>.')
        );
    }

    public static function renderSettingsFieldApiKey()
    {
        $options = get_option('wp_forum_connect', []);
        printf(
            '<input type="text" name="%s" value="%s" class="regular-text code" /><p class="description">%s</p>',
            esc_attr('wp_forum_connect[api_key]'),
            esc_attr($options['api_key'] ?? ''),
            \AudentioForumConnect::phrase('An XF API key, which can be generated under <code>ACP > Setup API keys</code>
                <span id="xf-api-key-desc-session_cookie"><br /><strong>XF Session Cookie</strong>: Key type: Super user key. Scopes: auth, node:read, thread:read, thread:write</span>
                <span id="xf-api-key-desc-wp_user"><br /><strong>Connected WP/XF users</strong>: Key type: Super user key. Scopes: auth, node:read, thread:read, thread:write</span>
                <span id="xf-api-key-desc-iframe"><br /><strong>Iframe</strong>: Key type: Guest key. Scopes: node:read, thread:read. <strong>User API keys with scope thread:write need to be created and entered for each WP user</strong></span>')
        );
    }

    public static function renderSetupFieldWPAPIKey()
    {
        $options = get_option('wp_forum_connect', []);
        if (empty($options['api_key'])) {
        ?>
        <div class="notice notice-info">
            <p><?php echo \AudentioForumConnect::phrase('If you are using Article and Forum Connect Plus with the XenForo addon, enter the below API key along with your WordPress URL in the XenForo settings, at ACP > Setup > Options > Article and Forum Connect')?></p>
        </div>
        <?php
        }
    }

    public static function renderSettingsFieldWPAPIKey()
    {
        $apiKey = get_option('wp_forum_connect_wp_api_key');

        if (!$apiKey) {
            $apiKey = self::generateApiKey();
        }

        printf('<strong>%s</strong>', $apiKey);

        printf(
            '<br/><input type="checkbox" id="wp_forum_connect_regen_api_key" name="wp_forum_connect_regen_api_key" /><label for="wp_forum_connect_regen_api_key">%s</label>',
            \AudentioForumConnect::phrase('Regenerate')
        );

        printf('<p class="description" id="wp_forum_connect_regen_api_key-description">%s</p>',
            \AudentioForumConnect::phrase("If using the XF counterpart addon, enter this in the Article and Forum Connect XenForo settings page, at ACP > Setup > Options > Article and Forum Connect
            <br/>You may regenerate it by checking the above checkbox and save this page, should the API key ever be compromised")
        );
    }

    public static function renderSettingsFieldWPAPIUser()
    {
        $options = get_option('wp_forum_connect', []);

        $users = wp_dropdown_users([
            'show_option_all' => '-- ' . \AudentioForumConnect::phrase('Select user') . ' --',
            'name' => 'wp_forum_connect[wp_api_user]',
            'id' => 'wp_forum_connect_wp_api_user',
            'selected' => $options['wp_api_user'] ?? '',
            'echo' => false,
        ]);

        printf(
            '%s<br /><label for="wp_forum_connect_wp_api_user">%s</label>',
            $users,
            \AudentioForumConnect::phrase('If using the XF counterpart addon, select the user to authenticate as when making requests from XF')
        );
    }

    public static function renderSettingsFieldCommentingMode()
    {
        $options = get_option('wp_forum_connect', []);
        printf(
            '<input type="radio" id="wp_forum_connect_commenting_mode_linked_threads" name="%s" value="linked_threads" %s /><label for="wp_forum_connect_commenting_mode_linked_threads">%s</label>
            <p class="description">%s</p><br />
            <input type="radio" id="wp_forum_connect_commenting_mode_hybrid" name="%s" value="hybrid" %s /><label for="wp_forum_connect_commenting_mode_hybrid">%s</label>
            <p class="description">%s</p><br />
            <input type="radio" id="wp_forum_connect_commenting_mode_auth_only" name="%s" value="auth_only" %s /><label for="wp_forum_connect_commenting_mode_auth_only">%s</label>
            <p class="description">%s</p><br />',

            esc_attr('wp_forum_connect[commenting_mode]'),
            $options['commenting_mode'] == 'linked_threads' ? 'checked' : '',
            \AudentioForumConnect::phrase('Linked threads'),
            \AudentioForumConnect::phrase('The comments section will only be handled by Article and Forum Connect on posts that are linked to a XenForo thread. Other posts will use the pre-existing comment functionality - this includes native WP comments, or any comment plugins you have installed, such as Disqus.'),

            esc_attr('wp_forum_connect[commenting_mode]'),
            $options['commenting_mode'] == 'hybrid' ? 'checked' : '',
            \AudentioForumConnect::phrase('Hybrid'),
            \AudentioForumConnect::phrase('The comments section will be handled by Article and Forum Connect on posts that are linked to a XenForo thread, and it will also handle authentication for comments on existing posts that are not linked to a thread, by using the forum user.<br />This will only provide authentication for native WP comments, and not for any 3rd party comment plugins.'),

            esc_attr('wp_forum_connect[commenting_mode]'),
            $options['commenting_mode'] == 'auth_only' ? 'checked' : '',
            \AudentioForumConnect::phrase('Auth-only'),
            \AudentioForumConnect::phrase('No posts will be linked to any XenForo threads, but Article and Forum Connect  will handle authentication for comments on existing posts, by using the forum user.<br />This will only provide authentication for native WP comments, and not for any 3rd party comment plugins.')
        );
    }

    public static function renderSettingsFieldIntegrationType()
    {
        $options = get_option('wp_forum_connect', []);
        printf(
            '<input type="radio" id="wp_forum_connect_integration_type_session_cookie" name="%s" value="session_cookie" %s /><label for="wp_forum_connect_integration_type_session_cookie">%s</label>
            <p class="description">%s</p><br />
            <input type="radio" id="wp_forum_connect_integration_type_wp_user" name="%s" value="wp_user" %s /><label for="wp_forum_connect_integration_type_wp_user">%s</label>
            <p class="description">%s</p><br />
            <input type="radio" id="wp_forum_connect_integration_type_iframe" name="%s" value="iframe" %s disabled /><label for="wp_forum_connect_integration_type_iframe">%s</label>
            <p class="description">%s</p><br />',

            esc_attr('wp_forum_connect[integration_type]'),
            $options['integration_type'] == 'session_cookie' ? 'checked' : '',
            \AudentioForumConnect::phrase('XF Session Cookie'),
            \AudentioForumConnect::phrase('Uses the XF session cookie to authenticate users.<br />Requires the blog and forum to be on the same domain.'),

            esc_attr('wp_forum_connect[integration_type]'),
            $options['integration_type'] == 'wp_user' ? 'checked' : '',
            \AudentioForumConnect::phrase('Connected WP/XF users'),
            \AudentioForumConnect::phrase('Connect an XF and WP user account together to authenticate users.<br />Supports the blog and forum being on different domains.'),

            esc_attr('wp_forum_connect[integration_type]'),
            $options['integration_type'] == 'iframe' ? 'checked' : '',
            \AudentioForumConnect::phrase('Iframe'),
            \AudentioForumConnect::phrase('Embeds the forum thread as an iframe.<br />Requires the counterpart XF addon and for the forum to be allowed to be embedded on the blog.<br />Cannot be used when in auth-only commenting mode')
        );
    }

    public static function renderSettingsFieldConnectedUserMethod()
    {
        $options = get_option('wp_forum_connect', []);
        printf(
            '<input type="radio" id="wp_forum_connect_connected_user_method_login" name="%s" value="login" %s /><label for="wp_forum_connect_connected_user_method_login">%s</label>
            <p class="description">%s</p><br />
            <input type="radio" id="wp_forum_connect_connected_user_method_oauth" name="%s" value="oauth" %s disabled /><label for="wp_forum_connect_connected_user_method_oauth">%s</label>
            <p class="description">%s</p><br />',

            esc_attr('wp_forum_connect[connected_user_method]'),
            $options['connected_user_method'] == 'login' ? 'checked' : '',
            \AudentioForumConnect::phrase('Login Credentials'),
            \AudentioForumConnect::phrase('A login form will be displayed in the comments section to authenticate with a XenForo account, which will then be linked to the WordPress user. Login credentials will not be stored and users will only need to provide them once.'),

            esc_attr('wp_forum_connect[connected_user_method]'),
            $options['connected_user_method'] == 'oauth' ? 'checked' : '',
            \AudentioForumConnect::phrase('Oauth / XenForo connected account'),
            \AudentioForumConnect::phrase('A connected accounts provider can use OAuth to link the accounts together. Requires the counterpart XF addon.')
        );
    }

    public static function renderSettingsFieldNativeComments()
    {
        $options = get_option('wp_forum_connect', []);

        printf(
            '<input type="checkbox" id="wp_forum_connect_native_comments" name="%s" %s disabled /><label for="wp_forum_connect_native_comments">%s</label> <p class="description">%s</p>',
            esc_attr('wp_forum_connect[native_comments]'),
            $options['native_comments'] ?? null ? 'checked' : '',
            \AudentioForumConnect::phrase('Use native WordPress comments'),
            \AudentioForumConnect::phrase('Instead of replacing the comments template with posts pulling from XenForo, you can restore the native WordPress comments template.<br />This requires the counterpart XF addon, and the setting to create WP comments from posts to be enabled.')
        );
    }

    public static function validateApiSettings($input)
    {
        \AudentioForumConnect::checkConnection(true, $input);

        return $input;
    }

    public static function renderSettingsFieldCreateThread()
    {
        $options = get_option('wp_forum_connect', []);

        $createThread = !empty($options['create_thread']);

        printf(
            '%s<br /><br /><select name="wp_forum_connect[create_thread]" id="xf-thread-create" class="postform">
                <option value="0" class="no-remove" %s>%s</option>
                <option class="no-remove" %s>%s</option>
            </select>',
            \AudentioForumConnect::phrase('Automatically create a new thread when a new post is published'),
            !$createThread ? 'selected' : '',
            \AudentioForumConnect::phrase('No'),
            $createThread ? 'selected' : '',
            \AudentioForumConnect::phrase('Yes')
        );
    }

    public static function renderSettingsFieldDefaultForum()
    {
        printf(
            '<label for="xf-thread-forum">%s</label>
            <select name="wp_forum_connect[default_forum]" id="xf-thread-forum" class="postform">
                <option value="" class="no-remove">-- %s --</option>
            </select>
            <br /><br />
            <label for="xf-thread-prefix">%s</label>
            <select name="wp_forum_connect[default_prefix]" id="xf-thread-prefix" class="postform">
                <option value="" class="no-remove">-- %s --</option>
            </select>
            <br /><br />%s',
            \AudentioForumConnect::phrase('XF forum'),
            \AudentioForumConnect::phrase('Select forum'),
            \AudentioForumConnect::phrase('XF prefix'),
            \AudentioForumConnect::phrase('Select prefix'),
            \AudentioForumConnect::phrase('Default forum details per category are configued in Posts > Categories')
        );
    }

    public static function renderSettingsFieldDefaultUser()
    {
        $options = get_option('wp_forum_connect', []);

        $users = wp_dropdown_users([
            'show_option_all' => '-- ' . \AudentioForumConnect::phrase('Select user') . ' --',
            'name' => 'wp_forum_connect[default_user]',
            'id' => 'wp_forum_connect_default_user',
            'selected' => $options['default_user'] ?? '',
            'echo' => false,
        ]);

        printf(
            '%s<br /><label for="wp_forum_connect_default_user">%s</label>',
            $users,
            \AudentioForumConnect::phrase('If no user is selected, the post author will be used')
        );
    }

    public static function renderSettingsFieldAlwaysUseDefaults()
    {
        $options = get_option('wp_forum_connect', []);

        printf(
            '<input type="checkbox" id="wp_forum_connect_always_use_defaults" name="%s" %s /><label for="wp_forum_connect_always_use_defaults">%s</label> <p class="description">%s</p>',
            esc_attr('wp_forum_connect[always_use_defaults]'),
            $options['always_use_defaults'] ?? null ? 'checked' : '',
            \AudentioForumConnect::phrase('Always use default thread settings'),
            \AudentioForumConnect::phrase('For a simpler experience, if your threads will always be created with the same configuration, you can hide the meta box on the post composer page.')
        );
    }

    public static function renderSettingsFieldHideArticleDiscussionLink()
    {
        $options = get_option('wp_forum_connect', []);

        printf(
            '<input type="checkbox" id="wp_forum_connect_hide_article_discussion_link" name="%s" %s /><label for="wp_forum_connect_hide_article_discussion_link">%s</label> <p class="description">%s</p>',
            esc_attr('wp_forum_connect[hide_article_discussion_link]'),
            $options['hide_article_discussion_link'] ?? null ? 'checked' : '',
            \AudentioForumConnect::phrase('Hide article discussion link'),
            \AudentioForumConnect::phrase('Hide the link at the end of the article to link to the discussion thread')
        );
    }

    public static function renderSettingsFieldEmailGravatar()
    {
        $options = get_option('wp_forum_connect', []);

        printf(
            '<input type="checkbox" id="wp_forum_connect_email_gravatar" name="%s" %s /><label for="wp_forum_connect_email_gravatar">%s</label> <p class="description">%s</p>',
            esc_attr('wp_forum_connect[email_gravatar]'),
            $options['email_gravatar'] ?? null ? 'checked' : '',
            \AudentioForumConnect::phrase('Use email address for gravatar'),
            \AudentioForumConnect::phrase('If a user has no avatar uploaded and no explicit gravatar set, try and use their email address with gravatar')
        );
    }

    public static function renderSettingsFieldPostThreadIdMetaKey()
    {
        $options = get_option('wp_forum_connect', []);

        printf(
            '<input type="text" name="%s" value="%s" class="regular-text code" /><p class="description">%s</p>',
            esc_attr('wp_forum_connect[post_thread_id_meta_key]'),
            esc_attr($options['post_thread_id_meta_key'] ?? ''),
            \AudentioForumConnect::phrase('The name of a legacy post meta field storing existing related XenForo thread IDs from a previous integration.<br />Intended for keeping linked IDs from a previous integration without having to migrate data.')
        );
    }

    public static function renderSettingsFieldCookiePrefix()
    {
        $options = get_option('wp_forum_connect', []);

        printf(
            '<input type="text" name="%s" value="%s" class="regular-text code" /><p class="description">%s</p>',
            esc_attr('wp_forum_connect[xf_cookie_prefix]'),
            esc_attr($options['xf_cookie_prefix'] ?? ''),
            \AudentioForumConnect::phrase('If your XenForo cookie prefix has been changed, enter it here')
        );
    }

    public static function renderSettingsFieldDefaultForumUserId()
    {
        $options = get_option('wp_forum_connect', []);

        printf(
            '<input type="text" name="%s" value="%s" class="regular-text code" /><p class="description">%s</p>',
            esc_attr('wp_forum_connect[default_forum_user_id]'),
            esc_attr($options['default_forum_user_id'] ?? ''),
            \AudentioForumConnect::phrase('This is only used when using <code>\AudentioForumConnect::get_xf_user()</code> or related functions (such as <code>xf_user_is_member_of</code>) manually in your WordPress theme.<br />If the visitor is not logged into a XF user account, the user with this user ID will be loaded instead. This can be used to set things like a default avatar or usergroups')
        );
    }

    public static function renderSettingsFieldSessionAuthClientKey()
    {
        $options = get_option('wp_forum_connect', []);

        printf(
            '<input type="text" name="%s" value="%s" class="regular-text code" /><p class="description">%s</p>',
            esc_attr('wp_forum_connect[session_auth_client_key]'),
            esc_attr($options['session_auth_client_key'] ?? ''),
            \AudentioForumConnect::phrase('If you have the Audentio API XF addon installed, you can authenticate with the XF API directly with the session cookie.<br />This means your XenForo API Key can be a more restrictive Guest key (with scopes node:read, thread:read, thread:write) instead of a Super-user key.<br />User API keys with scope thread:write need to be created and entered for each WP user.<br />Create a Session Client at ACP > Setup > [Audentio] API > Authentication > Session authentication, with scopes thread:read, thread:write')
        );
    }

    public static function renderSettingsFieldAutoOpenIframe()
    {
        $options = get_option('wp_forum_connect', []);

        printf(
            '<input type="checkbox" id="wp_forum_connect_auto_open_iframe" name="%s" %s /><label for="wp_forum_connect_auto_open_iframe">%s</label> <p class="description">%s</p>',
            esc_attr('wp_forum_connect[auto_open_iframe]'),
            $options['auto_open_iframe'] ?? null ? 'checked' : '',
            \AudentioForumConnect::phrase('Auto-open iframe'),
            \AudentioForumConnect::phrase('Whether or not the iframe should automatically open when viewing a post')
        );
    }

    public static function renderSettingsFieldAutoOpenIframeQueryString()
    {
        $options = get_option('wp_forum_connect', []);

        printf(
            '<input type="text" name="%s" value="%s" class="regular-text code" /><p class="description">%s</p>',
            esc_attr('wp_forum_connect[auto_open_iframe_query_string]'),
            esc_attr($options['auto_open_iframe_query_string'] ?? ''),
            \AudentioForumConnect::phrase('A query string value that will automatically open the iframe. For example, if passing through <em>?comments=1</em> as a query string value, enter <em>comments=1</em>')
        );
    }

    protected static function generateApiKey()
    {
        $newApiKey = sha1(time() . get_bloginfo('name'));
        update_option('wp_forum_connect_wp_api_key', $newApiKey);
        return $newApiKey;
    }

    public static function resetWPAPIKey($status)
    {
        if($status == 'on') {
            self::generateApiKey();
        }
    }

    public static function renderSettingsPage()
    {
        wp_enqueue_script('wp_forum_connect_admin_options', plugins_url('article-forum-connect/public/js/options.js'), ['jquery'], false);
        wp_enqueue_script('wp_forum_connect_admin_thread_config', plugins_url('article-forum-connect/public/js/thread-config.js'), ['jquery'], false, true);

        $option = get_option('wp_forum_connect', []);

        $AudentioForumConnectBridgeTitle = \AudentioForumConnect::phrase('Article and Forum Connect');
        ?>
        <h2><?php echo $AudentioForumConnectBridgeTitle ?></h2>
        <script>
        var wpForumConnectForumId = '<?php echo $option['default_forum'] ?? null?>';
        var wpForumConnectPrefixId = '<?php echo $option['default_prefix'] ?? null?>';
        </script>
        <?php if (!empty($option['xf_url'])) { ?>
        <div id="wp_forum_connect_version_free" class="notice notice-warning" style="display:none">
            <p><?php echo \AudentioForumConnect::phrase('You are currently using the free version of Article and Forum Connect. To enable premium features, install the counterpart XF addon, and enter the WordPress URL and API Key (found below) at ACP > Setup > Options > [Aud] Article and Forum Connect. <a href="javascript:window.location.reload()">Check settings</a>')?></p>
        </div>
        <div id="wp_forum_connect_version_plus" class="notice notice-success" style="display:none">
            <p><?php echo \AudentioForumConnect::phrase('You are currently using the premium version of Article and Forum Connect.')?></p>
        </div>
        <?php } ?>
        <form action="options.php" method="post">
            <?php

            settings_fields('article-forum-connect');
            do_settings_sections('article-forum-connect');

            submit_button();
            ?>
        </form>
        <?php
    }
}