<?php

namespace AudentioForumConnect\Actions;

use AudentioForumConnect\Client;
use AudentioForumConnect\HookInterface;
use AudentioForumConnect\HookTrait;

/**
 * Class AdminMenu
 * @package AudentioForumConnect\Actions
 */
class TransitionPostStatus implements HookInterface
{
    use HookTrait;

    /**
     *
     */
    public static function callback($new_status, $old_status, $post)
    {
        $option = get_option('wp_forum_connect');

        $postId = $post->ID;

        // don't run unless publishing or it's not a post
        if ($post->post_status != 'publish' || $post->post_type != 'post') {
            return;
        }

        if (!\AudentioForumConnect::should_create_thread($postId)) {
            return;
        }

        $threadId = \AudentioForumConnect::get_post_thread_id($postId);

        // don't run if there's already a thread
        if ($threadId) {
            return;
        }

        $message = get_post_meta($postId, 'xf_excerpt', true) ?: get_the_excerpt($postId);

        if (!$message) {
            return;
        }

        if ($promotedThreadId = get_post_meta($postId, 'xf_promoted_thread_id', true)) {

            \AudentioForumConnect::update_post_thread_id($postId, $promotedThreadId);
            delete_post_meta($postId, 'xf_promoted_thread_id');

            return;
        }

        $userId = get_post_meta($postId, 'xf_user_id', true);
        if (!$userId) {

            if (!empty($option['default_user'])) {
                $userId = $option['default_user'];
            } else {
                $userId = $post->post_author;
            }
        }

        $forumId = get_post_meta($postId, 'xf_forum_id', true);
        $prefixId = get_post_meta($postId, 'xf_prefix_id', true);
        if (!$forumId) {
            $forumId = $option['default_forum'] ?? null;
            $prefixId = $option['default_prefix'] ?? null;
        }
        if (!$forumId) {

            $categories = wp_get_post_categories($postId);

            foreach ($categories as $category) {
                if ($forumId = get_term_meta($category, 'xf_forum_id', true)) {
                    $prefixId = get_term_meta($category, 'xf_prefix_id', true);
                    continue;
                }
            }
        }
        if (!$forumId) {
            return;
        }

        $client = \AudentioForumConnect::AudentioForumConnectClientForThread($userId);

        $title = html_entity_decode(get_the_title($postId));

        if (strlen($title) > 150) {
            $title = substr($title, 0, 147) . '...';
        }

        $data = [
            'wp_post_id' => $postId,
        ];

        if ($prefixId) {
            $data['prefix_id'] = $prefixId;
        }

        $message .= "\n\n" . '[URL=' . get_permalink($post) . ']' . \AudentioForumConnect::phrase('Makalenin tamamına bakın...') . '[/URL]';
        $message = html_entity_decode($message);

        $thread = $client->xf->thread->create($forumId, $title, $message, $data);

        \AudentioForumConnect::update_post_thread_id($postId, $thread->thread_id);
    }
}